@echo off
@rem ==========================================================================
@rem == Licensed Materials - Property of IBM Corporation
@rem == (C) Copyright IBM Corporation 2015
@rem ==========================================================================
@rem Deploys SPE patch resources
@rem Run this from an SPE console window. This is a WINDOWS ONLY script.
@rem ==========================================================================
@rem
if not defined InSPEConsole echo This command must be run in an SPE console session
if not defined InSPEConsole goto :EOF

Setlocal EnableDelayedExpansion

rem Generate timestamp for backup dir
for /f "delims=" %%a in ('wmic OS Get localdatetime  ^| find "."') do set dt=%%a
set TIMESTAMP=D%dt:~0,8%.T%dt:~8,6%

set PATCH_JAR=
set MESSAGE=Invalid option %1

:READ_COMMAND_LINE_PARMS
if "%1"=="" goto VALIDATE_PATCH_JAR

rem read in -j
if /i "%1"=="-j" (
rem read in the -j value: \path\to\patch.jar
    set PATCH_JAR=%2
    shift
    goto READ_NEXT
)

rem help syntax
if /i "%1"=="/?" goto SYNTAX

echo %MESSAGE%
echo.
goto SYNTAX

:READ_NEXT
shift
goto READ_COMMAND_LINE_PARMS

:VALIDATE_PATCH_JAR
echo PATCH_JAR=%PATCH_JAR%
set MESSAGE=Missing mandatory parameter -j
if not defined PATCH_JAR goto ERROR
set MESSAGE=SPE patch jar file does not exist: %PATCH_JAR%
if not exist %PATCH_JAR% goto ERROR
rem extract the last four characters from the jar name
set JAR_EXT=%PATCH_JAR:~-4%
rem remove the last character. it might be a double quote
set JAR_EXT=%JAR_EXT:~0,-1%
set MESSAGE=Invalid jar file name extension: %JAR_EXT%
rem if jar path was specified with double quotes
if /i "%JAR_EXT%"=="jar" goto VALIDATE_OK
rem if jar path was not specified with double quotes
if /i "%JAR_EXT%"==".ja" goto VALIDATE_OK
goto ERROR

:VALIDATE_OK
rem Generate WORK_DIR name and extract jar to it
for %%j in (%PATCH_JAR%) do (
    Set JAR_NAME=%%~nxj
)
echo JAR_NAME=%JAR_NAME%
set NO_JAR_NAME=%JAR_NAME:~0,-4%
set WORK_DIR="%SPE_INSTALL_DIR%\%NO_JAR_NAME%"
echo NO_JAR_NAME=%NO_JAR_NAME%
echo WORK_DIR=%WORK_DIR%
set MESSAGE=Failed to remove %WORK_DIR%
if exist %WORK_DIR% rd /s /q %WORK_DIR%
if ERRORLEVEL 1 goto ERROR

echo.
echo Extracting files...
set MESSAGE=Failed to extract %PATCH_JAR% to %WORK_DIR%
java com.ibm.spe.core.utils.ExtractJarFile -name=%PATCH_JAR% -output=%WORK_DIR%
if ERRORLEVEL 1 goto ERROR
set MESSAGE=Failed to change directory to %WORK_DIR%
cd %WORK_DIR%
if ERRORLEVEL 1 goto ERROR

:PROCESS_OBSOLETE_FILES
if exist OBSOLETE_FILES (
    cd "%SPE_INSTALL_DIR%"    
    set OBSOLETE_DIR=obsolete
    if not exist !OBSOLETE_DIR! (
        echo.
        echo Creating !OBSOLETE_DIR! directory
        set MESSAGE=Failed to create !OBSOLETE_DIR! directory
        md !OBSOLETE_DIR!
        if ERRORLEVEL 1 goto ERROR
    )

    cd %WORK_DIR%
    echo.
    echo Moving files to the !OBSOLETE_DIR! directory...
    set MESSAGE=Failed to move files to !OBSOLETE_DIR! folder
    for /f "tokens=1* delims=]" %%i in ('find /n /v ""^<OBSOLETE_FILES') do (
        set FILE=%%j
        set FILE=!FILE:/=\!
        set SOURCE_FILE=%SPE_INSTALL_DIR%\!FILE!
        set TARGET_FILE=%SPE_INSTALL_DIR%\!OBSOLETE_DIR!\!FILE!
        if exist !SOURCE_FILE! (
            echo obsolete !SOURCE_FILE!
            set MESSAGE=Failed to copy !File! to !OBSOLETE_DIR! folder
            for %%F in ("!TARGET_FILE!") do set DIR_NAME=%%~dpF
            if not exist "!DIR_NAME!" md "!DIR_NAME!"
            move /y "!SOURCE_FILE!" "!DIR_NAME!" 
            if ERRORLEVEL 1 goto ERROR
        )
    )
    if ERRORLEVEL 1 goto ERROR 
)

:CREATE_BACKUP_DIR
cd "%SPE_INSTALL_DIR%"
set BACKUP_DIR=backups\%TIMESTAMP%_PATCH
if not exist %BACKUP_DIR% (
    echo.
    echo Creating %BACKUP_DIR% directory
    set MESSAGE=Failed to create %BACKUP_DIR% directory
    md %BACKUP_DIR%
    if ERRORLEVEL 1 goto ERROR
)

:DEPLOY_PATCH_FILES
cd %WORK_DIR%
echo.
echo Removing accessory files...
if exist OBSOLETE_FILES del /q /f /s OBSOLETE_FILES
if exist CVS rd /q /s CVS
if exist META-INF rd /q /s META-INF

echo.
echo Creating directories...
set MESSAGE=Failed to create directories
for /d /r %%d in (*) do (
    set line=%%d
    if not exist "%SPE_INSTALL_DIR%!line:%CD%=!" (
        set MESSAGE=Failed to create "%SPE_INSTALL_DIR%!line:%CD%=!"
        md "%SPE_INSTALL_DIR%!line:%CD%=!"
        if ERRORLEVEL 1 goto ERROR         
    )
)
if ERRORLEVEL 1 goto ERROR 

rem Get list of files to copy 
set LIST_OF_FILES="%SPE_INSTALL_DIR%\%NO_JAR_NAME%.txt"
if exist %LIST_OF_FILES% del /q /f /s %LIST_OF_FILES%
for /r %%f in (*) do (
    set line=%%f
    echo .!line:%CD%=! >> %LIST_OF_FILES%
)

echo.
echo Deploying patch files...
set FILE=
set TARGET_FILE=
set MESSAGE=Failed to deploy files
for /f "tokens=1* delims=]" %%i in ('find /n /v ""^<%LIST_OF_FILES%') do (
    set FILE=%%j
    set FILE=!FILE:~0,-1!
    set TARGET_FILE=%SPE_INSTALL_DIR%\!FILE!
    set BACKUP_FILE=%SPE_INSTALL_DIR%\%BACKUP_DIR%\!FILE!   
    set MESSAGE=Failed to make a backup copy of !TARGET_FILE!
    if exist !TARGET_FILE! (
        echo backing up old !FILE!
        for %%F in ("!BACKUP_FILE!") do set DIR_NAME=%%~dpF
        xcopy /i/q/y "!TARGET_FILE!" "!DIR_NAME!"
        if ERRORLEVEL 1 goto ERROR    
    )    
    set MESSAGE=Failed to deploy !FILE!
    echo deploying !FILE!
    copy /y "!FILE!" "!TARGET_FILE!"
    if ERRORLEVEL 1 goto ERROR 
)
if ERRORLEVEL 1 goto ERROR 

:DONE
set MESSAGE=
cd %SPE_INSTALL_DIR%
if exist %WORK_DIR% rd /s /q %WORK_DIR%
if exist %LIST_OF_FILES% del /q /f %LIST_OF_FILES%
echo.
echo SPE patch resources were deployed successfully
echo.
goto :EOF

:ERROR
echo.
echo.%MESSAGE%
echo.
cd %SPE_INSTALL_DIR%
if exist %WORK_DIR% rd /s /q %WORK_DIR%
if exist %LIST_OF_FILES% del /q /f %LIST_OF_FILES%
goto :EOF

:SYNTAX
echo Deploys SPE patch resources
echo.
echo speinstall-patch.bat -j \path\to\patch.jar 
echo.
echo Command line arguments:
echo -j  (Required) Path to the SPE patch jar file
echo.
goto :EOF

:EOF
